<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 */
if (!defined("mxMainFileLoaded")) die ("You can't access this file directly...");

/**
 * HTML-Manipulation
 * 
 * @class pmxDom
 * @author Sven Hedstroem-Lang <svenhlang@gmail.com>
 * @copyright 2022-03
 * @since pragmaMx 2.8.1
 * @phpversion >=8.0.1
 * @api https://jqueryui.com
 * @api ICONS https://api.jqueryui.com/theming/icons/
 * @example admin.php?op=dom
 * @require
 *      jQuery
 *      jQueryUI
 *      pmxHeader::add_jquery()
 *      pmxHeader::add_script_code_docu()
 *      pmxHeader::add_script_code_func()
 *      pmxHeader::getUniqId()
 *      pmxCookie
 *      MX_IS_ADMIN
 *      _REDIRECTMESS2
 * @files
 *      includes/classes/Dom/css/progressbar.css
 *      includes/classes/Dom/js/progressbar.js
 * 
 * Alle Funktionen sind exakt gleich aufgebaut
 *      function(array $config, array $options)
 *      $config - werden die Elemente zusammengebaut und betreffen nur PMX
 *      $options - betreffen nur die Api
 * 
 * TODO:
 * # https://jqueryui.com/accordion/#sortable - wär das nicht was für unsere Module ? -> admin.php?op=modules
 * # https://wiki.selfhtml.org/wiki/HTML/Tutorials/Listen/Gestaltung_mit_CSS#Eine_Liste_als_Zeitstrahl
 * # pmxHeader::getUniqId() hierher
 * # alle scripte in eine eigene datei statt in html
 * # table with different color - like UI
 * 
 */
class pmxDom {
    
    /**
     * @const _DEFAULT_PREFIX
     * @default "_"
     * @copyright pmx 2.8.1
     */
    const _DEFAULT_PREFIX = "_";

    /**
     * To add jQueryUI Theme-CSS
     * 
     * @const _DEFAULT_SETTHEME 
     * @default true
     * @copyright pmx 2.8.1
     */
    const _DEFAULT_SETTHEME = true;
    
    /**
     * @var string $_PREFIX
     * @default self::_DEFAULT_PREFIX
     * @copyright pmx 2.8.1
     */
    private static $_PREFIX = self::_DEFAULT_PREFIX;

    /**
     * @var bool $_SET_THEME
     * @default self::_DEFAULT_SETTHEME
     * @copyright pmx 2.8.1
     */
    private static $_SET_THEME = self::_DEFAULT_SETTHEME;

    
    /**
     * INFO: some elements require jQuerUI-Theme
     * 
     * @constructor
     * @private - verhindert die direkte Erzeugung des Objektes
     * @copyright pmx 2.8.1
     */
    private function __construct() {}

    /**
     * pmxDom::setPrefix()
     * To create the uniqId
     * 
     * INFO: optional no need
     * 
     * @param string $prefix
     * @return void
     * @example
     *      pmxDom::setPrefix("any");
     *      // => set $_PREFIX = "any_1"
     * @copyright pmx 2.8.1
     */
    public function setPrefix($prefix)
    {
        self::$_PREFIX = self::isValidUniqId($prefix) ? $prefix : self::_DEFAULT_PREFIX;
    }

    /**
     * pmxDom::setTheme()
     * Hier kann expliziet überschrieben werden, ob die jQueryUI-CSS eingebunden werden soll oder nicht. zb im Theme
     * Standart ist das Theme immer eingebunden
     * 
     * @param bool [$bool=true] - optional - true added the jqueryui-CSS-File
     * @return void
     * @copyright pmx 2.8.1
     */
    public static function setTheme($bool = self::_DEFAULT_SETTHEME)
    {
        self::$_SET_THEME = $bool;
    }

    /**
     * pmxDom::addMenu()
     * 
     * INFO: upgrade jQuery - geht mit dem alten jQueryUI nicht
     * 
     * INFO: Begrenzt auf 5 Untermenüs ($runs)
     * 
     * @api https://jqueryui.com/menu/
     * @since pmx 2.8.1
     * @param array $config
     * @param string [$config["uniqId"]] Optional
     * @param array $config["items"]
     * @param string $config["items"][["title"]] require
     * @param string $config["items"][["icon"]]
     * @param array $config["items"][["styleClass"]] Like Category add "ui-widget-header" AND/OR disable "ui-state-disabled"
     * @param string $config["items"][["url"]]
     * @param array [$options] API - Optional
     * @return string
     * @copyright pmx 2.8.1
     * @example TODO: public
     *      echo pmxDom::addMenu(["items" => [
     *          ["title" => "menu1", "icon" => "", "styleClass" => [], "url" => "index.php"],
     *          ["title" => "menu2", "icon" => "", "styleClass" => [], "items" => [
     *              ["title" => "menu21", "icon" => "", "styleClass" => []],
     *              ["title" => "menu22", "icon" => "", "styleClass" => []],
     *          ]]
     *      ]]);
     */
    private static function _addMenu($config, $options = [])
    {
        self::_checkTheme();
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        pmxHeader::add_jquery('ui', 'ui/jquery.ui.menu.js');
        pmxHeader::add_script_code_func('$("#' . $config["uniqId"] . '").menu(' . json_encode($options) . ');');

        /**
         * ## WARNING START ##
         * 
         * INFO: Begrenzt auf 5 Untermenüs ($runs)
         * 
         * Damit kann man beliebig viele Untermenüs erzeugen
         * Diese Funktion wird immer wieder aufgerufen - Deshalb ein try/catch erforderlich
         * 
         * @param array $arr The $config["items"] to create the menu
         * @param bool $needULTag true implements <ul></ul>
         * @param int $runs Check the runs of the function
         * @return string
         * @copyright pmx 2.8.1
         */
        function _getMenuItems($arr, $needULTag, $runs)
        {
            try {
                if ($runs <= 5) {
                    $_out = [$needULTag === true ? '<ul>' : ""];
                    foreach ($arr as $value) {
                        $_styleClass = "";
                        $_icon = "";
                        if (array_key_exists("icon", $value)) {
                            $_icon = '<span class="ui-icon ' . $value["icon"] . '"></span>';
                        }
                        if (array_key_exists("styleClass", $value)) {
                            $_styleClass = ' class="' . implode(" ", $value["styleClass"]) . '"';
                        }
                        $_out[] = '<li' . $_styleClass . '>';
                        if (array_key_exists("url", $value)) {
                            $_out[] = '<a href="' . $value["url"] . '"><div>' . $_icon . $value["title"] . '</a></div>';
                        } else {
                            $_out[] = '<div>' . $_icon . $value["title"] . '</div>';
                        }
                        // ### Gibt es Untermenüs ?
                        if (array_key_exists("items", $value) && is_array($value["items"])) {
                            $_out[] = _getMenuItems($value["items"], true, $runs++);
                        }
                        $_out[] = '</li>';
                    }
                    $_out[] = $needULTag === true ? '</ul>' : "";
                    return implode("", $_out);
                } else {
                    trigger_error("Maximale Unter-Menüs erreicht > " . $runs, E_USER_NOTICE);
                }
            } catch (Exception $e) {
                trigger_error($e->getMessage(), E_USER_NOTICE);
            }
            return "";
        }// ## WARNING END ##
        
        return '<ul id="' . $config["uniqId"] . '">' . _getMenuItems($config["items"], false, 0) . '</ul>';
    }

    /**
     * pmxDom::addButton()
     * INFO: Button is created by <button>
     * 
     * @param array $config
     * @param string [$config["uniqId"]] - Optional
     * @param string $config["text"]
     * @param string [$config["type"]="button"] - "button", "submit", "reset"
     * @param string [$config["styleClass"]]
     * @param string [$config["value"]]
     * @param array [$options=[]] - Optional
     * @return string
     * @copyright pmx 2.8.1
     * @example
     *      echo pmxDom::addButton(["text" => "Click me", "styleClass" => "ui-button ui-widget ui-corner-all"]);
     */
    public static function addButton($config, $options = [])
    {
        self::_checkTheme();
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        $config["type"] = array_key_exists("type", $config) ? $config["type"] : null;
        $config["styleClass"] = array_key_exists("styleClass", $config) ? $config["styleClass"]: "";
        $config["value"] = array_key_exists("value", $config) ? $config["value"] : "";
        pmxHeader::add_jquery('ui', 'ui/jquery.ui.button.js');
        pmxHeader::add_script_code_func('$("#' . $config["uniqId"] . '").button(' . json_encode($options) . ');');
        switch ($config["type"]) {
            case "submit":
            case "reset":
            case "button":
                break;
            default:
                $config["type"] = "button";
                break;
        }
        return '<div class="widget">' . implode("", [
            '<button id="' . $config["uniqId"] . '"',
            ' type="' . $config["type"] . '"',
            ' class="' . $config["styleClass"] . '"',
            ' value="' . $config["value"] . '">',
            $config["text"] . '</button>'
        ]) . '</div>';
    }

    /**
     * pmxDom::addSlider()
     * 
     * @api https://jqueryui.com/slider/
     * @param array $config
     * @param string [$config["uniqId"]] - Optional
     * @param array [$options] - API - Optional
     * @return string
     * @copyright pmx 2.8.1
     * @example
     *      echo pmxDom::addSlider([], ["value" => 25]);
     */
    public static function addSlider($config, $options = [])
    {
        self::_checkTheme();
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        pmxHeader::add_jquery('ui', 'ui/jquery.ui.slider.js');
        pmxHeader::add_script_code_func('$("#' . $config["uniqId"] . '").slider(' . json_encode($options) . ');');
        return '<div id="' . $config["uniqId"] . '"></div>';
    }

    /**
     * pmxDom::addDatepicker()
     * 
     * create an <input>
     * 
     * @api https://jqueryui.com/datepicker/
     * @param array [$config] - Optional
     * @param string [$config["uniqId"]] - Optional
     * @param string [$config["type"]] - type="" - Optional
     * @param string [$config["name"]] - name="" - Optional
     * @param string [$config["value"]] - value="" - Optional
     * @param array [$options] - API - Optional
     * @return string
     * @copyright pmx 2.8.1
     * @example
     *      echo pmxDom::addDatepicker(["value" => "Add your Birthday"]);
     */
    public static function addDatepicker($config = [], $options = [])
    {
        self::_checkTheme();
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        pmxHeader::add_jquery('ui', 'ui/jquery.ui.datepicker.js', 'ui/i18n/jquery.ui.datepicker-' . _DOC_LANGUAGE . '.min.js');
        pmxHeader::add_script_code_func('$("#' . $config["uniqId"] . '").datepicker(' . json_encode($options) . ');');        
        return implode("", [
            '<input id="' . $config["uniqId"] . '"',
            array_key_exists("type", $config) ? ' type="' . $config['type'] . '"' : "",
            array_key_exists("name", $config) ? ' name="' . $config['name'] . '"' : "",
            array_key_exists("value", $config) ? ' value="' . $config['value'] . '"' : "",
            '>'
        ]);
    }
    
    /**
     * pmxDom::addSelectMenu()
     * 
     * @param array $config
     * @param string $config["uniquId"]
     * @param string $config["selectName"] - name of select-tag
     * @param array $config["items"] - An Array or titles
     * @param array [$options] - API - Optional
     * @return string
     * @copyright pmx 2.8.1
     * @example TODO: public
     */
    private static function addSelectMenu($config, $options = [])
    {
        /*
        self::_checkTheme();
        if (self::_isValidUniqId($config["uniqId"]) && $config["items"]) {
            // TODO: not exists > pmxHeader::add_jquery('ui', 'ui/jquery.ui.selectmenu.js');
            pmxHeader::add_script_code_func(
                '$("#' . $config["uniqId"] . '").selectmenu(' . json_encode($options) . ');'
            );
            $_out = [
                '<select id="' . $config["uniqId"] . '" name="' . $config["selectName"] . '">',
            ];
            foreach ($config["items"] as $value) {
                $_out[] = '<option>' . $value . '</option>';
            }
            $_out[] = '</select>';
            return implode("", $_out) ;
        }
        */
        return "";
    }
    
    /**
     * pmxDom::scrollToTop()
     * ersetzt jquery.scrollUp.min.js
     * 
     * @param array $config
     * @param string $config["uniqId"]
     * @param int $config["milliseconds"]
     * @param array [$options=[]] - API - Optional
     * @param string $options=["scrollTop"]
     * @param int [$milliseconds=400]
     * @return void
     * @copyright pmx 2.8.1
     * @example
     *      pmxDom::scrollTo(["uniqId" => "top", "milliseconds" => 2500], ["scrollTop" => 0]);
     */
    public static function scrollTo($config, $options = [])
    {
        $options = is_array($options) ? $options : [];
        pmxHeader::add_jquery();
        pmxHeader::add_script_code_docu(
            '$("#' . $config["uniqId"] . '").click(function() {
                $("html, body, document").animate(' . json_encode($options) . ', ' . $config["milliseconds"] . ');
              });'
        );
    }

    /**
     * pmxDom::addDialog()
     * ersetzt lightbox.js, pmxHeader::add_lightbox()
     * Erzeugt den nur den Dialog, nicht das Element zum Anklicken
     * 
     * INFO: Effects: blind, bounce, clip, drop, explode, fade, fold, highlight, puff, pulsate, scale, shake, size, slide, transfer
     * 
     * @api https://api.jqueryui.com/dialog/
     * @param string $config["uniqId"] - require
     * @param string $config["title"]
     * @param string $config["text"]
     * @param string $config["img"] - Path to image
     * @param array|bool [$options=true] - Optional - Use an Array with own $options OR true with default $options OR false is empty $options
     * @param string [$options["width"]]
     * @param string [$options["height"]]
     * @param array [$options["show"]]
     * @param string [$options["show"]["effect"]]
     * @param int [$options["show"]["duration"]] - 2000 are 2 seconds
     * @param string [$options["hide"]["effect"]]
     * @param int [$options["hide"]["duration"]] - 2000 are 2 seconds
     * @return string
     * @copyright pmx 2.8.1
     * @example TODO:
     *      // HTML
     *      <div id="clickMe">Click Me</div>
     *      // PHP
     *      echo pmxDom::addDialog(["uniqId" => "clickMe", "title" => "the title dialog", "text" => "the text dialog"]);
     */
    public static function addDialog($config, $options = true)
    {
        self::_checkTheme();
        pmxHeader::add_jquery('ui', 'ui/jquery.ui.dialog.js');
        switch ($options) {
            case is_array($options):
                break;
            case $options === false:
                $options = [];
                break;
            case $options === true:
            default:
                $options = [
                    "autoOpen" => false, // require false
                    "modal" => true, // require true
                    "closeText" => "" // tooltip
                ];
                break;
        }

        $_img = array_key_exists("img", $config) ? '<img src="' . $config["img"] . '">' : "";
        $_outputId = "_" . $config["uniqId"];

        pmxHeader::add_script_code_func(
            '$("#' . $_outputId . '").dialog(' . json_encode($options) . ');
            $("#' . $config["uniqId"] . '").click(function() {
                $("#' . $_outputId . '").dialog("open");
            });'
        );
        return '<div id="' . $_outputId . '" title="' . $config["title"] . '">' . $_img . $config["text"] . '</div>';
    }
    
    /**
     * pmxDom::addTooltip()
     * 
     * @param array $config
     * @param string $config["uniqId"] - Diese wird nicht überprüft, da hier auch mehrere ID's verwendet werden können getrennt durch ein Komma
     * @param array [$options=[]] - API - Optional
     * @return void
     * @copyright pmx 2.8.1
     * @example
     *      pmxDom::addTooltip(["uniqId" => "#title, .anyClassName, #anotherOne"]);
     */
    public static function addTooltip($config, $options = [])
    {
        self::_checkTheme();
        pmxHeader::add_jquery('ui', 'ui/jquery.ui.tooltip.js');
        pmxHeader::add_script_code_func('$("' . $config["uniqId"] . '").tooltip(' . json_encode($options) . ');');
        // TODO: der tooltip geht manchmal nicht - Theme "admin-pmx" macht probleme
    }

    /**
     * pmxDom::addTabs()
     * 
     * TODO: https://jqueryui.com/tabs/#vertical
     * 
     * @api https://jqueryui.com/tabs/
     * @param array $config
     * @param string [$config["uniqId"]] Optional
     * @param array $config["items" => array(["title1", "text1"], ["title2", "text2"])] Wenn das Array leer ist, wird auch nichts ausgegeben
     * @param string [cookie] - optional - use any cookie name
     * @param array [$options] API - Optional
     * @return string Show with echo pmxDom::addTabs()
     * @copyright pmx 2.8.1
     * @example
     *      echo pmxDom::addTabs(["items" => array(["title 1", "text 1"], ["title 2", "text 2"], ["title 3", "text 3"])]);
     */
    public static function addTabs($config, $options = [])
    {
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        $config["items"] = array_key_exists("items", $config) ? $config["items"] : array();
        $config["cookie"] = array_key_exists("cookie", $config) ? $config["cookie"] : "";

        $_cookie_on = ';';
        if (strlen($config["cookie"]) > 0) {
            // SET cookie with JS
            $_cookie_on = '.on("tabsactivate",function(event,ui){
            let tab = ui.newTab.parent().children().index(ui.newTab);
            document.cookie = "' . $config["cookie"] . '=" + tab + "";
            });';
            // GET cookie with PHP
            $_phpCookie = isset($_COOKIE[htmlspecialchars($config["cookie"])]) ? htmlspecialchars($_COOKIE[$config["cookie"]]) : "";
            $options = array_merge(
                $options,
                // ["active" => pmxCookie::get($config["cookie"])] // error in PHP 5.6
                ["active" => $_phpCookie]
            );
        }

        if (count($config["items"]) > 0) {
            // FIX jQueryUI height in theme admin-pmx
            // pmxHeader::add_script_code_docu('$(".ui-tabs-nav, .ui-tabs-panel").css("float", "left").css("width", "-webkit-fill-available");');

            self::_checkTheme();
            pmxHeader::add_jquery('ui', 'ui/jquery.ui.tabs.js');
            pmxHeader::add_script_code_func('$("#' . $config["uniqId"] . '" ).tabs(' . json_encode($options) . ')' . $_cookie_on);

            $_out = ['<div id="' . $config["uniqId"] . '"><ul>'];
            $_text = [];
            foreach ($config["items"] as $key => $value) {
                $_id = $config["uniqId"] . '-' . ($key + 1);
                $_out[] = '<li><a href="#' . $_id . '">' . $value[0] . '</a></li>';
                $_text[] = '<div id="' . $_id . '"><p>' . $value[1] . '</p></div>';
            }
            $_out[] = '</ul>' . implode("", $_text) . '</div>';       
            return implode("", $_out);
        }
        // kein Inhalt
        return "";
    }

    /**
     * pmxDom::addAccordion()
     * 
     * @api https://jqueryui.com/accordion/
     * @param array $config
     * @param string [$config["uniqId"]] - Optional
     * @param array [$options] - API - optional
     * @return string
     * @copyright pmx 2.8.1
     * @example
     *      echo pmxDom::addAccordion(["items" => array(array("1 Title", "1 Lorem Ipsum"), array("2 title", "2 Lorem Ipsum"))],
     *      ["collapsible" => true, "icons" => ["header" => "ui-icon-circle-arrow-e", "activeHeader" => "ui-icon-circle-arrow-s"]]);
     */
    public static function addAccordion($config, $options = [])
    {
        self::_checkTheme();
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        pmxHeader::add_jquery('ui', 'ui/jquery.ui.accordion.js');
        pmxHeader::add_script_code_func('$("#' . $config["uniqId"] . '").accordion(' . json_encode($options) . ');');
        $_out = ['<div id="' . $config["uniqId"] . '">'];

        foreach ($config["items"] as $value) {
            $_out[] = '<h3>' . $value[0] . '</h3>
            <div><p>' . $value[1] . '</p></div>';
        }
        $_out[] = "</div>";

        return implode("", $_out);
    }

    /**
     * pmxDom::addProgressbar()
     * 
     * @api https://jqueryui.com/progressbar/
     * @require includes/classes/Dom/css/progressbar.css
     * @require includes/classes/Dom/js/progressbar.js
     * TODO: Unter 30 Sekunden lädt unendlich - try/catch
     * @param array $config
     * @param string [$config["uniqId"]] - optional
     * @param int $config["seconds"]
     * @param string $config["url"]
     * @param bool $config["addStyle"] - Add style when set to true like see @example
     * @param array|bool [$options=true] - array to set own $options OR false to set empty $options OR true to set with optimal default $options
     * @param array $config["ready"] - Require when
     * @param int $config["ready"][0 => 85] - Require when $options=true - (85 = 85%)
     * @param string $config["ready"][1 => ""] - Require when $options=true
     * @param string $config["more"] => ""] - Can add more Text behind
     * @param [$options=true] - API - Optional
     * @return string
     * @copyright pmx 2.8.1
     * @example
     *      echo pmxDom::addProgressbar(["seconds" => 50, "url" => "index.php",
     *          "ready" => [85, "Die Einstellungen wurden gespeichert!"], "addStyle" => true], true),
     *      // CSS STYLE - $config["addStyle" => true]
     *      pmxHeader::add_style_code(
     *          '.ui-progressbar { position: relative; }
     *          .progress-label {
     *              position: absolute;
     *              left: 50%;
     *              top: 50%;
     *              transform: translate(-50%, -50%);
     *              font-weight: bold;
     *          }'
     *      );
     */
    public static function addProgressbar($config, $options = true)
    {
        self::_checkTheme();
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        $config["seconds"] = array_key_exists("seconds", $config) ? $config["seconds"] * 2 : 40 * 2;
        $config["url"] = array_key_exists("url", $config) ? $config["url"] : "";
        $config["addStyle"] = array_key_exists("addStyle", $config) ? $config["addStyle"] : false;
        $config["more"] = array_key_exists("more", $config) ? $config["more"] : "";

        if ($config["seconds"] >= 30) {
            pmxHeader::add_jquery('ui', 'ui/jquery.ui.progressbar.js');
            if ($config["addStyle"] === true) {
                pmxHeader::add_style('includes/classes/Dom/css/progressbar.css');
            }

            if (is_array($options) || $options === false) {
                $options = is_array($options) ? $options : [];
                pmxHeader::add_script_code_docu(
                    '$("#' . $config["uniqId"] . '").progressbar(' . json_encode($options) . ');'
            );
            }
            // default option
            else if ($options === true) {
                // $options = [];
                pmxHeader::add_script('includes/classes/Dom/js/progressbar.js');
                pmxHeader::add_script_code_docu(
                    'addDomProgressbar( $("#' . $config["uniqId"] . '"), "' . $config["url"] . '", ' . json_encode($config["ready"]) . ', ' . $config["seconds"] . ');'
                );
            }
            
            $_out = [
                '<div id="' . $config["uniqId"] . '">',
                    '<div class="progress-label">Loading...</div>',
                '</div>',
                '<h4 id="progress-ready" style="text-align:center;">&nbsp;</h4>',
                $config["more"]
            ];

            if (MX_IS_ADMIN && strlen($config["url"]) > 0) {
                $_out[] = '<p style="text-align:center;">(
                <a href="' . $config["url"] . '">' . _REDIRECTMESS2 . '</a>
                )</p>';
            }
            return implode("", $_out);
        } else {
            trigger_error("Please set the seconds of progressbar >= 30", E_USER_NOTICE);
        }
        if (strlen($config["url"]) > 0) {
            header('Refresh:' . $config["seconds"] . '; url=' . $config["url"] . '');
        }
        return "";
    }

    /**
     * pmxDom::addAutocomplete()
     * 
     * TODO: geht mit dem alten jQueryUI nicht, added: jQuerUI 1.8
     * 
     * @param array $config
     * @param string [$config["uniqId"]] - Optional
     * @param array $options
     * @return string
     * @copyright pmx 2.8.1
     * @example TODO: hier fehlen noch <form> elemente, update jquery, public
     */
    private static function _addAutocomplete($config, $options)
    {
        self::_checkTheme();
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        pmxHeader::add_jquery('ui', 'ui/jquery.ui.autocomplete.js');
        pmxHeader::add_script_code_func(
            '$("#' . $config["uniqId"] . '").autocomplete(' . json_encode($options) . ');'
        );
        return '<input id="' . $config["uniqId"] . '">';
    }

    /**
     * pmxDom::addCountDown()
     * 
     * INFO: without jQuery
     * 
     * @param array $config
     * @param string [$config["uniqId"]] - Optional
     * @param int [$config["seconds"]=3]
     * @param int [$config["url"]=""]
     * @return string
     * @copyright pmx 2.8.1
     * @example
     *      // HTML
     *      <div>Something doing in <p id="myId">15</p> seconds ...</div>
     *      // PHP
     *      pmxDom::addCountDown(["uniqId" => "myId" ,"seconds" => 15, "url" => "admin.php"]);
     *      // OR - %0 Platzhalter für die Zeit
     *      pmxDom::addCountDown(["seconds" => 15, "url" => "admin.php", "text" => "Something doing in %0 seconds"]);
     */
    public static function addCountDown($config)
    {
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        $config["seconds"] = array_key_exists("seconds", $config) ? $config["seconds"] : 3;
        $config["url"] = array_key_exists("url", $config) ? $config["url"] : "";
        $config["text"] = array_key_exists("text", $config) ? $config["text"] : "";
        if ($config["seconds"] > 0) {
            pmxHeader::add_script('includes/classes/Dom/js/countdown.js');
            pmxHeader::add_script_code_docu(
                'addDomCountDown("' . $config["uniqId"] . '", ' . ($config["seconds"] - 1) . ', "' . $config["url"] . '");'
            );

            if (is_string($config["text"])) {
                return str_replace(
                    "%0",
                    '<nobr id="' . $config["uniqId"] . '">' . $config["seconds"] . '</nobr>',
                    $config["text"]
                );
            }
        }
        if ($config["url"] !== "") {
            header('Refresh:' . $config["seconds"] . '; url=' . $config["url"] . '');
        }
        return "";
    }

    /**
     * pmxDom::addScrollContent()
     * Erzeugt ein <div> zum scrollen
     * 
     * INFO: without jQuery
     * 
     * @param array $config
     * @param string [$config["uniqId"]] - Optional
     * @param string $config["text"]
     * @param string [$config["width"=>"100%"]] - optional - Kann auch in andere CSS Formate angegeben werden z.b. px
     * @param string [$config["height"=>"100px"]] - optional
     * @return string
     * @copyright pmx 2.8.1
     */
    public static function addScrollContent($config)
    {
        $config["uniqId"] = array_key_exists("uniqId", $config) ? $config["uniqId"] : self::getUniqId();
        $config["text"] = array_key_exists("text", $config) ? $config["text"] : "";
        $config["width"] = array_key_exists("width", $config) ? $config["width"] : "100%";
        $config["height"] = array_key_exists("height", $config) ? $config["height"] : "100px";
        return implode("", [
            '<div id="' . $config["uniqId"] . '" ',
            'style="overflow:scroll;width:' . $config["width"] . ';height:' . $config["height"] . ';overflow-x:hidden">',
            $config["text"],
            '</div>'
        ]);
    }

    /**
     * TODO:
     */
    public static function _addMenuHover()
    {
        self::_checkTheme();
        /* 1
        pmxHeader::add_style_code('
a {
  text-decoration: none;
}
a:hover {
  text-decoration: underline;
}
.dropdown {
    position: relative;
    display: block;
    float: left;
    padding-right: 10px;
    margin-right: 15px;
}
.dropdown.active > a {
  color: green;
}
.dropdown-menu {
    width: 100%;
    top: 33px;
    min-width: 300px;
    box-shadow: 0 10px 10px rgba(0, 0, 0, .15);
    background: #f7f7f7;
    height: auto;
    position: absolute;
    top: 100%;
    left: 0px;
    display: none;
    float: left;
}

.dropdown-menu .dropdown-wrapper {
    position: relative;
}
.category-menu a {
    display: block;
  }
');
*/

pmxHeader::add_jquery();
/* menu 1
pmxHeader::add_script_code_docu('
    $(".main-nav ul > li").mouseenter(function(e){
        $(this).addClass("active").find(".dropdown-menu").show();
        }).mouseleave(function(e){
        $(this).removeClass("active").find(".dropdown-menu").hide();            
    }); 
    $(".dropdown-menu .close").click( function(e){
        e.stopPropagation();
        $(".dropdown-menu").hide();
        $(".dropdown .dropdown-toggle").parent().removeClass("active");
        console.log("close btn!");
    });
');
*/

/* 1
$_menu = '<li class="dropdown">
<a href="#" class="toggle">Category 1
<div class="dropdown-menu">
  <div class="dropdown-wrapper">
  <!-- close button <button class="close"></button> -->
    <div class="category-menu">
      <a href="#"><img src="modules/News/images/submissions.png"> Subcategory 1</a>
      <a href="#"><img src="modules/Documents/images/document.png"> Subcategory 2</a>
      <a href="#"><img src="modules/Encyclopedia/images/encyclopedia.png"> Subcategory 3</a>
    </div>
  </div>
</div>
</li>';
*/

// menu2
/*
# img
# CSS
# JS - USER
# JS - BACKEND
# menu overlay
*/

/* 2
pmxHeader::add_script_code_func('
//$(".ui-menuhover li").hover();
$(".ui-menuhover li")
.mouseenter(function() {
    $(this).find(".ui-menuhover-dropdown")
    .animate({height: "toggle"})
    //.show("slow")
    ;
})
.mouseleave(function() {
    $(this).find(".ui-menuhover-dropdown")
    .animate({height: "toggle"})
    //.hide("fast")
    ;
});

$(".ui-menuhover-item").hover(
    function() {
        $(this).addClass("ui-state-hover");
    }, function() {
        $(this).removeClass("ui-state-hover");
    }
);
');*/
/* 2
pmxHeader::add_style_code('
.ui-menuhover {
    //padding: 5px 0 5px 0;
    //margin-bottom: 0;
    //height: 50px;
    width: 80%;
}
.ui-menuhover li {
    //display: flex;
    //justify-content:space-around;
    //list-style-type: none;

    //display: inline;
    //margin: 0 10px 0 0;
    //padding: 10px;
    position: relative;
    display:inline
}
.ui-menuhover-item {
}


// das verstecken un per JS einbinden
// |: untermenu > ul > li :|
.ui-menuhover-item-wrapper {
}

.ui-menuhover-item-wrapper:hover { // > ui-state-active
    //background-color: green;
}
.ui-menuhover-dropdown {
    //top: 80px;
    //left: 159px;
    display: none;
}

.ui-menuhover-item-img {
    // vertical-align:middle;
}

');
*/

/* 2
// ul class
// ui-accordion-header ui-helper-reset ui-state-default ui-accordion-header-active ui-state-active ui-corner-top ui-accordion-icons
function _getMenu($id) {
    return '
<li class="ui-helper-reset ui-widget-content ui-state-default ui-menuhover-item">
    <div class="ui-helper-reset ui-state-default ui-menuhover-item-wrapper" aria-haspopup="true" id="ui-id-'.$id.'" tabindex="-1" role="menuitem">
        <span class="ui-helper-reset ui-icon ui-icon-caret-1-e"></span>
        <a href="#">
            <img src="modules/Documents/images/document.png">
            Category ui-id-'.$id.'
        </a>
    </div>
    <!-- SUB CATEGORIE -->
    <ul class="ui-helper-reset ui-widget ui-widget-header ui-front ui-menuhover-dropdown" aria-hidden="false" role="menu" aria-expanded="true">
        <li class="ui-helper-reset ui-widget-content ui-state-default ui-menuhover-item" aria-disabled="true">
            <div id="ui-id-1" tabindex="-1" role="menuitem" class="ui-menuhover-item-wrapper">
                <a href="#">
                    <img src="modules/News/images/submissions.png">
                    Sub Category ui-id-1
                </a>
            </div>
        </li>
        <li class="ui-helper-reset ui-widget-content ui-state-default ui-menuhover-item">
            <div id="ui-id-2" tabindex="-1" role="menuitem" class="ui-helper-reset ui-menuhover-item-wrapper">
                <a href="#">
                    <img src="modules/Documents/images/document.png">
                    Sub Category ui-id-2
                </a>
            </div>
        </li>
        <li class="ui-helper-reset ui-widget-content ui-state-default ui-menuhover-item">
            <div id="ui-id-3" tabindex="-1" role="menuitem" class="ui-helper-reset ui-menuhover-item-wrapper">
                <a href="#">
                    <img src="modules/Encyclopedia/images/encyclopedia.png">
                    Sub Category ui-id-3
                </a>
            </div>
        </li>
    </ul>
</li>
';
}
*/

// 3

/*
https://www.sassmeister.com/

.ui-menuhover {
    display: blocK;
    .ui-menuhover-header {
        margin: 0;
        padding: 0.15%;
        height: 3em;
        width: 80%;
        text-align: left;
        &> .ui-menuhover-item {
            height: 100%;
            margin-right: 0.5em;
            padding: 0 1em;
            background:#201f5f;
            width: 100px;
            
            &> a {
                height: 3em;
                color: #ffffff;
                line-height: 3;
            }
        }
        &> .ui-menuhover-item-sub {
            position: relative;
            ul {
                margin: 0;
                padding: 0.5em 0;
                background: #000000;
                position: absolute;
                top: -1000em;
                left: -160px;
                width: 80px;
                li a {
                    height: 100%;
                    display: inline;
                    float: left;
                    padding: 0.4em;
                    color: #fff;
                }
            }
            &:hover ul {
                top: 3em;
            }
        }
        li {
            display:inline-block;
        }
    }
}


*/
pmxHeader::add_style_code('
@charset "utf-8";
.ui-menuhover {
    display: blocK;
        
    width: 100%
  }
  .ui-menuhover .ui-menuhover-header {
    margin: 0;
    padding: 1em;
    height: 3em;
  }
  .ui-menuhover .ui-menuhover-header > .ui-menuhover-item {
    height: 100%;
    padding: 0 1em;
    background: #ccc;

    -moz-border-radius: 0.5em 0.5em 0 0;
    border-radius: 0.5em 0.5em 0 0;
  }
  .ui-menuhover .ui-menuhover-header > .ui-menuhover-item > .ui-menuhover-item-wrapper {
    height: 3em;
    color: #fff;
    line-height: 3;

    margin: 0 2em 0 2em;
  }
  .ui-menuhover .ui-menuhover-header > .ui-menuhover-item-sub {
    position: relative;
  }
  .ui-menuhover .ui-menuhover-header > .ui-menuhover-item-sub ul {
    margin: 0;
    padding: 0.5em 0;
    background: #000;
    position: absolute;
    top: -1000em;
    left: 0;

    width: 100%;
  }
  .ui-menuhover .ui-menuhover-header > .ui-menuhover-item-sub ul li {

    word-break: normal;
    white-space: normal;
    word-break: break-all;
  }
  .ui-menuhover .ui-menuhover-header > .ui-menuhover-item-sub ul li .ui-menuhover-item-wrapper {
    height: 100%;
    display: inline;
    float: left;
    padding: 0.1em;
    color: #fff;


    width: 100%;
    left: -10px;
  }
  .ui-menuhover .ui-menuhover-header > .ui-menuhover-item-sub:hover ul {
    top: 3em; /* höhe von ui-menuhover-item-wrapper */
  }
  .ui-menuhover .ui-menuhover-header li {
    display: inline-block;

    white-space: normal;
  }
  .ui-menuhover .ui-menuhover-item-wrapper { /* alle li > div */
  }
  .ui-menuhover-img {
    padding-left: 29px;
    background-position: left center;
    background-repeat: no-repeat;
    background-size: 24px 24px;
  }
  /* ui-menuhover-item + ui-menuhover-item-wrapper = gleiche */
');
function _getMenu3() {
    // ui-menuhover-item-sub need ?
    return '
<div id="uniqId-menuhover" class="ui-helper-reset ui-menuhover">
    <ul class="ui-helper-reset ui-menuhover-header">
        <li class="ui-helper-reset ui-menuhover-item"><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#"><img src="modules/Encyclopedia/images/encyclopedia.png"> Menu 1</a></div></li>
        <li class="ui-helper-reset ui-menuhover-item ui-menuhover-item-sub"><div class="ui-helper-reset ui-menuhover-item-wrapper">
            <a href="#">
            <span class="ui-menuhover-img" style="background-image: url(modules/Documents/images/document.png);">Erweiterungen</span>
            <ul>
                <li class="ui-helper-reset ui-menuhover-item"><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Sub 2-1</a></div></li>
                <li class="ui-helper-reset ui-menuhover-item"><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Sub 2-2</a></div></li>
            </ul>
            </div>
        </li>
        <li class="ui-helper-reset ui-menuhover-item ui-menuhover-item-sub"><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Sub Menu 3</a>
            <ul>
                <li><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Sub 3-1</a></div></li>
                <li><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Sub 3-2</a></div></li>
            </ul>
            </div>
        </li>
        <li class="ui-helper-reset ui-menuhover-item"><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Menu 4</a></div></li>
        <li class="ui-helper-reset ui-menuhover-item ui-menuhover-item-sub"><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Sub Menu 5</a>
            <ul>
                <li><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Sub 5-1</a></div></li>
                <li><div class="ui-helper-reset ui-menuhover-item-wrapper"><a href="#">Sub 5-2 TextText TextText TextText</a></div></li>
            </ul>
            </div>
        </li>
    </ul>
</div>

<div>TODO: jquerycss, img, text zu lang BREAK, padding -> margin, wenn menu größer als sub - werden sub (-> 100%) nebeneinander angezeigt statt untereinander</div>
    ';
}

return _getMenu3();


// 2 return '<ul id="menuhover" class="ui-helper-reset ui-widget ui-widget-header ui-menuhover" role="menu" tabindex="0" aria-activedescendant="ui-id-1">' . _getMenu(1) . _getMenu(2) . _getMenu(3).'</ul>' . '<button onclick="window.location.reload();">Refresh Page</button>';
// 1 return '<div class="main-nav"><ul>' . $_menu . $_menu . $_menu . $_menu . '</ul></div>';      
    }

    private static function _addExample($config, $options)
    {
        return implode("", $config , $options);
    }
    
    /**
     * pmxDom::getUniqId()
     * Get an uniqId
     * 
     * @alias pmxHeader::getUniqId()
     * @return string - to lowerCase
     * @copyright pmx 2.8.1
     */
    public static function getUniqId()
    {
        $_id = strtolower(pmxHeader::getUniqId(self::$_PREFIX));
        if (self::isValidUniqId($_id)) {
            return $_id;
        }
        return strtolower(pmxHeader::getUniqId(self::_DEFAULT_PREFIX));
    }

    /**
     * pmxDom::isValidUniqId()
     * 
     * Check if uniqId valide - Require for CSS-Style
     * Require regex(a-zA-Z_) as first string
     * All after regex([a-zA-Z0-9_])
     * 
     * @param string $uniqId
     * @return bool
     * @copyright pmx 2.8.1
     */
    public static function isValidUniqId($uniqId)
    {
        if (preg_match('/^[a-zA-Z_]{1}[a-zA-Z0-9_]+$/i', $uniqId) === 1) {
            return true;
        }
        trigger_error("isValidUniqId() invalide uniqId > " . $uniqId, E_USER_NOTICE);
        return false;
    }

    // ###########
    // # PRIVATE #
    // ###########

    /**
     * self::_checkTheme()
     * 
     * Wird nur für jQuerUI benötigt
     * @return void
     */
    private static function _checkTheme()
    {
        if (self::$_SET_THEME === true && pmxBase::juitheme()) {
            pmxHeader::add_style('layout/jquery/ui/' . pmxBase::juitheme() . '/jquery-ui.min.css');
        }
    }

}

?>
